#include <stdio.h>
#include <stdlib.h>
#include <fat.h>

#include "screen.h"
#include "sound.h"

static void *xfb = NULL;
static GXRModeObj *rmode = NULL;

void initConsole()
{
	// Init default video
	VIDEO_Init();
	
	// Obtain the preferred video mode from the system
	// This will correspond to the settings in the Wii menu
	rmode = VIDEO_GetPreferredMode(NULL);

	// Allocate memory for the display in the uncached region
	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	
	// Initialise the console, required for printf
	console_init(xfb,20,20,rmode->fbWidth,rmode->xfbHeight,rmode->fbWidth*VI_DISPLAY_PIX_SZ);
	
	// Set up the video registers with the chosen mode
	VIDEO_Configure(rmode);
	
	// Tell the video hardware where our display memory is
	VIDEO_SetNextFramebuffer(xfb);
	
	// Make the display visible
	VIDEO_SetBlack(FALSE);

	// Flush the video register changes to the hardware
	VIDEO_Flush();

	// Wait for Video setup to complete
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE) VIDEO_WaitVSync();

	printf("\nStarting DragonMedia Player...");
}

int main(int argc, char **argv) 
{
	// Initialize console (testing only)
	//initConsole();
	
	// Initialize fat
	fatInitDefault();
	
	// Instantiate and initialize screen
	Screen display;
	
	// Instantiate and initialize sound
	Sound sound;

	// Main loop
	bool done = false;
	
	while(!done) 
	{
		// Process next frame
		done = display.update();
		sound.update();
	}

	return 0;
}
